@@ -1,5 +1,5 @@
 package Photonic::LE::NP::EpsL;
-$Photonic::LE::NP::EpsL::VERSION = '0.015';
+$Photonic::LE::NP::EpsL::VERSION = '0.016';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::LE::NP::EpsL
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 COPYRIGHT NOTICE
 
@@ -70,7 +70,7 @@ determines the precision of the results. It defaults to 1e-7.
 
 =back
 
-=head1 ACCESORS (read only)
+=head1 ACCESSORS (read only)
 
 =over 4
 
@@ -112,8 +112,9 @@ check.
 use namespace::autoclean;
 use PDL::Lite;
 use PDL::NiceSlice;
-use PDL::Complex;
 use Photonic::LE::NP::AllH;
+use Photonic::Utils qw(lentzCF);
+use List::Util qw(min);
 use Photonic::Types;
 use Moose;
 use MooseX::StrictConstructor;
@@ -124,15 +125,15 @@ has 'smallH'=>(is=>'ro', isa=>'Num', required=>1, default=>1e-7,
     	    documentation=>'Convergence criterium for Haydock coefficients');
 has 'smallE'=>(is=>'ro', isa=>'Num', required=>1, default=>1e-7,
     	    documentation=>'Convergence criterium for use of Haydock coeff.');
-has 'epsA'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_epsA',
+has 'epsA'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_epsA',
     documentation=>'Dielectric function of host');
-has 'epsB'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_epsB',
+has 'epsB'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_epsB',
         documentation=>'Dielectric function of inclusions');
-has 'u'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_u',
+has 'u'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_u',
     documentation=>'Spectral variable');
 
 has 'nr' =>(is=>'ro', isa=>'Photonic::LE::NP::AllH', required=>1);
-has 'epsL'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_epsL');
+has 'epsL'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_epsL');
 has 'nhActual'=>(is=>'ro', isa=>'Num', init_arg=>undef,
                  writer=>'_nhActual');
 has 'converged'=>(is=>'ro', isa=>'Num', init_arg=>undef, writer=>'_converged');
@@ -140,40 +141,17 @@ has 'converged'=>(is=>'ro', isa=>'Num', init_arg=>undef, writer=>'_converged');
 sub BUILD {
     my $self=shift;
     $self->nr->run unless $self->nr->iteration;
-    my $as=$self->nr->as;
-    my $b2s=$self->nr->b2s;
-    # Continued fraction evaluation: Lentz method
-    # Numerical Recipes p. 171
-    my $tiny=1.e-30;
-    my $converged=0;
+    my $as=pdl(map r2C($_), @{$self->nr->as})->cplx;
+    my $b2s=pdl(map r2C($_), @{$self->nr->b2s})->cplx;
+    my $min= min($self->nh, $self->nr->iteration);
 #    b0+a1/b1+a2/...
 #	lo debo convertir a
 #	a0-b1^2/a1-b2^2/
 #	entonces bn->an y an->-b_n^2
-    my $fn=$as->[0];
-    $fn=r2C($tiny) if $fn->re==0 and $fn->im==0;
-    my $n=1;
-    my ($fnm1, $Cnm1, $Dnm1)=($fn, $fn, r2C(0)); #previous coeffs.
-    my ($Cn, $Dn); #current coeffs.
-    my $Deltan;
-    while($n<$self->nh && $n<$self->nr->iteration){
-	$Dn=$as->[$n]-$b2s->[$n]*$Dnm1;
-	$Dn=r2C($tiny) if $Dn->re==0 and $Dn->im==0;
-	$Cn=$as->[$n]-$b2s->[$n]/$Cnm1;
-	$Cn=r2C($tiny) if $Cn->re==0 and $Cn->im==0;
-	$Dn=1/$Dn;
-	$Deltan=$Cn*$Dn;
-	$fn=$fnm1*$Deltan;
-	last if $converged=$Deltan->approx(1, $self->smallE)->all;
-	$fnm1=$fn;
-	$Dnm1=$Dn;
-	$Cnm1=$Cn;
-	$n++;
-    }
+    my ($fn, $n)=lentzCF($as, -$b2s, $min, $self->smallE);
     #If there are less available coefficients than $self->nh and all
     #of them were used, there is no remaining work to do, so, converged
-    $converged=1 if $self->nr->iteration < $self->nh;
-    $self->_converged($converged);
+    $self->_converged($n<$min || $self->nr->iteration<=$self->nh);
     $self->_nhActual($n);
     $self->_epsL($fn);
 }
