@@ -1,5 +1,5 @@
 package Photonic::Roles::Geometry;
-$Photonic::Roles::Geometry::VERSION = '0.015';
+$Photonic::Roles::Geometry::VERSION = '0.016';
 
 =encoding UTF-8
 
@@ -38,9 +38,9 @@ use Moose::Role;
 
 use PDL::Lite;
 use PDL::NiceSlice;
-use PDL::MatrixOps;
 use PDL::Complex;
 use Photonic::Types;
+use Photonic::Utils qw(any_complex lu_decomp);
 use Carp;
 use constant PI=>4*atan2(1,1);
 
@@ -232,8 +232,8 @@ sub _build_cUnitPairs {
     my @cpairs;
     for my $i(0..$nd-1){ #build pairs of vectors
 	for my $j($i+1..$nd-1){
-	    my $vc=($units->[$i]+i*$units->[$j]);
-	    my $vcn=sqrt(($units->[$i]+i*$units->[$j])->Cabs2->sumover);
+	    my $vc=($units->[$i]+i()*$units->[$j]);
+	    my $vcn=sqrt($vc->Cabs2->sumover);
 	    my $vp=$vc*(1/$vcn);
 	    push @cpairs, $vp;
 	}
@@ -267,16 +267,14 @@ sub _build_unitDyads {
 
 sub _build_unitDyadsLU {
     my $self=shift;
-    my ($lu, $perm, $parity) = lu_decomp($self->unitDyads);
-    die 'Unit Dyad not invertible' unless defined $lu;
-    return [($lu, $perm, $parity)];
+    [lu_decomp($self->unitDyads->r2C)];
 }
 
 sub _G0 {
     my $self=shift;
     my $value=shift;
     croak "Direction0 must be ".$self->ndims."-dimensional vector" unless
-	[$value->dims]->[0]==$self->ndims and $value->ndims==1;
+	$value->dim(0)==$self->ndims and $value->ndims==1;
     croak "Direction must be non-null" unless $value->inner($value)>0;
     my $arg=":". (",(0)" x $self->ndims); #:,(0),... dimension of space times
     $value=$value->norm; #normalize
@@ -292,12 +290,8 @@ sub Vec2LC_G { #longitudinal component of 'complex' vector field in
     my $field=shift; # vector field to project
     croak "Can't project unless Direction0 is set" unless
 	$self->has_Direction0;
-    my $iscomplex=ref $field eq 'PDL::Complex';
-    $field=$field->complex unless $iscomplex;
     my $gnorm=$self->GNorm;
-    my $result=Cscale($field, $gnorm)->sumover;
-    $result=$result->real unless $iscomplex;
-    return $result;
+    ($field * $gnorm->r2C)->sumover;
 }
 
 sub LC2Vec_G { #longitudinal vector field from its longitudinal
@@ -306,15 +300,11 @@ sub LC2Vec_G { #longitudinal vector field from its longitudinal
     my $field=shift; # scalar field of longitudinal components
     croak "Can't project unless Direction0 is set" unless
 	$self->has_Direction0;
-    my $iscomplex=ref $field eq 'PDL::Complex';
-    $field=$field->complex unless $iscomplex;
     my $gnorm=$self->GNorm;
     #$gnorm is XorY nx, ny...
     #$field is RoI nx ny nz
     #$result RoI XoY nx ny nz
-    my $result=$field->(,*1)*$gnorm;
-    $result=$result->real unless $iscomplex;
-    return $result;
+    $field->(,*1)*$gnorm;
 }
 
 no Moose::Role;
@@ -328,7 +318,7 @@ Photonic::Roles::Geometry
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 SYNOPSIS
 
@@ -342,7 +332,7 @@ version 0.015
 =item (for developers)
 
     package Photonic::Geometry::FromB;
-    $Photonic::Geometry::Geometry::VERSION = '0.015';
+    $Photonic::Geometry::Geometry::VERSION = '0.016';
     use namespace::autoclean;
     use Moose;
     has 'B' =>(is=>'ro', isa=>'PDL', required=>1,
@@ -357,7 +347,7 @@ Roles consumed by geometry objects to be used in a Photonic
 calculation. See also the specific implementations under
 L<Photonic::Geometry>.
 
-=head1 ACCESORS (defined by the implementation)
+=head1 ACCESSORS (defined by the implementation)
 
 =over 4
 
@@ -367,7 +357,7 @@ The characteristic function as PDL
 
 =back
 
-=head1 ACCESORS (read write)
+=head1 ACCESSORS (read write)
 
 =over 4
 
@@ -377,7 +367,7 @@ Direction of the zero length wavevector
 
 =back
 
-=head1 ACCESORS (read only)
+=head1 ACCESSORS (read only)
 
 =over 4
 
