@@ -1,5 +1,5 @@
 package Photonic::WE::R2::EpsilonP;
-$Photonic::WE::R2::EpsilonP::VERSION = '0.015';
+$Photonic::WE::R2::EpsilonP::VERSION = '0.016';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::WE::R2::EpsilonP
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 COPYRIGHT NOTICE
 
@@ -82,7 +82,7 @@ NOTE: Only works along principal directions.
 
 =back
 
-=head1 ACCESORS (read only)
+=head1 ACCESSORS (read only)
 
 =over 4
 
@@ -99,19 +99,13 @@ The macroscopic dielectric projection of the last operation
 
 use namespace::autoclean;
 use PDL::Lite;
-use PDL::NiceSlice;
-use PDL::Complex;
-use PDL::MatrixOps;
-#use Storable qw(dclone);
-#use PDL::IO::Storable;
-#use Photonic::WE::R2::AllH;
 use Photonic::Types;
 use Moose;
 use MooseX::StrictConstructor;
 
 extends 'Photonic::WE::R2::WaveP';
 
-has 'epsilon' =>  (is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
+has 'epsilon' =>  (is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef,
              writer=>'_epsilon',
              documentation=>'Wave projection from last evaluation');
 
