@@ -1,5 +1,5 @@
 package Photonic::WE::R2::GreenS;
-$Photonic::WE::R2::GreenS::VERSION = '0.015';
+$Photonic::WE::R2::GreenS::VERSION = '0.016';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::WE::R2::GreenS
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 COPYRIGHT NOTICE
 
@@ -80,7 +80,7 @@ symmetric.
 
 =back
 
-=head1 ACCESORS (read only)
+=head1 ACCESSORS (read only)
 
 =over 4
 
@@ -135,14 +135,11 @@ fraction. 0 means don't check.
 
 use namespace::autoclean;
 use PDL::Lite;
-use PDL::NiceSlice;
-use PDL::Complex;
-use PDL::MatrixOps;
-use Storable qw(dclone);
-use PDL::IO::Storable;
 use Photonic::WE::R2::AllH;
 use Photonic::WE::R2::GreenP;
 use Photonic::Types;
+use Photonic::Utils qw(tensor make_haydock make_greenp);
+use List::Util qw(all);
 use Moose;
 use MooseX::StrictConstructor;
 
@@ -168,11 +165,11 @@ has 'smallH'=>(is=>'ro', isa=>'Num', required=>1, default=>1e-7,
     	    documentation=>'Convergence criterium for Haydock coefficients');
 has 'smallE'=>(is=>'ro', isa=>'Num', required=>1, default=>1e-7,
     	    documentation=>'Convergence criterium for use of Haydock coeff.');
-has 'epsA'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_epsA',
+has 'epsA'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_epsA',
     documentation=>'Dielectric function of host');
-has 'epsB'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_epsB',
+has 'epsB'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_epsB',
         documentation=>'Dielectric function of inclusions');
-has 'u'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef, writer=>'_u',
+has 'u'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef, writer=>'_u',
     documentation=>'Spectral variable');
 
 with 'Photonic::Roles::KeepStates',  'Photonic::Roles::UseMask';
@@ -182,60 +179,18 @@ sub evaluate {
     $self->_epsA(my $epsA=$self->metric->epsilon->r2C);
     $self->_epsB(my $epsB=shift);
     $self->_u(my $u=1/(1-$epsB/$epsA));
-    my @greenP; #array of Green's projections along different directions.
-    my $converged=1;
-    foreach(@{$self->greenP}){
-	push @greenP, $_->evaluate($epsB);
-	$converged &&=$_->converged;
-    }
-    $self->_converged($converged);
-    my $reGreenP=PDL->pdl([map {$_->re} @greenP]);
-    my $imGreenP=PDL->pdl([map {$_->im} @greenP]);
-    my ($lu, $perm, $parity)=@{$self->geometry->unitDyadsLU};
-    my $reGreen=lu_backsub($lu, $perm, $parity, $reGreenP);
-    my $imGreen=lu_backsub($lu, $perm, $parity, $imGreenP);
-    my $nd=$self->ndims;
-    my $greenTensor=PDL->zeroes(2, $nd, $nd)->complex;
-    my $n=0;
-    for my $i(0..$nd-1){
-	for my $j($i..$nd-1){
-	    $greenTensor->(:,($i),($j)).=$reGreen->($n)+i*$imGreen->($n);
-	    $greenTensor->(:,($j),($i)).=$reGreen->($n)+i*$imGreen->($n);
-	    ++$n;
-	}
-    }
+    $self->_converged(all { $_->converged } @{$self->greenP});
+    my $greenTensor = tensor(pdl([map $_->evaluate($epsB), @{$self->greenP}])->complex, $self->geometry->unitDyadsLU, $self->geometry->ndims, 2);
     $self->_greenTensor($greenTensor);
-    return $greenTensor;
+    $greenTensor;
 }
 
 sub _build_haydock { # One Haydock coefficients calculator per direction0
-    my $self=shift;
-    my @haydock;
-    # This must change if G is not symmetric
-    foreach(@{$self->geometry->unitPairs}){
-	my $m=dclone($self->metric); #clone metric, to be safe
-	my $e=$_->r2C; #polarization
-	#Build a corresponding Photonic::WE::R2::AllH structure
-	my $haydock=Photonic::WE::R2::AllH->new(
-	    metric=>$m, polarization=>$e, nh=>$self->nh,
-	    keepStates=>$self->keepStates, smallH=>$self->smallH,
-	    reorthogonalize=>$self->reorthogonalize,
-	    use_mask=>$self->use_mask,
-	    mask=>$self->mask);
-	push @haydock, $haydock;
-    }
-    return [@haydock]
+    make_haydock(shift, 'Photonic::WE::R2::AllH');
 }
 
 sub _build_greenP {
-    my $self=shift;
-    my @greenP;
-    foreach(@{$self->haydock}){
-	my $g=Photonic::WE::R2::GreenP->new(
-	    haydock=>$_, nh=>$self->nh, smallE=>$self->smallE);
-	push @greenP, $g;
-    }
-    return [@greenP]
+    make_greenp(shift, 'Photonic::WE::R2::GreenP');
 }
 
 __PACKAGE__->meta->make_immutable;
