@@ -1,5 +1,5 @@
 package Photonic::WE::S::GreenP;
-$Photonic::WE::S::GreenP::VERSION = '0.015';
+$Photonic::WE::S::GreenP::VERSION = '0.016';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::WE::S::GreenP
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 COPYRIGHT NOTICE
 
@@ -74,7 +74,7 @@ value of the  dielectric functions of the particle $epsB.
 
 =back
 
-=head1 ACCESORS (read only)
+=head1 ACCESSORS (read only)
 
 =over 4
 
@@ -119,10 +119,11 @@ Criteria of convergence. 0 means don't check.
 
 use namespace::autoclean;
 use PDL::Lite;
-use PDL::NiceSlice;
 use PDL::Complex;
 use Photonic::WE::S::AllH;
 use Photonic::Types;
+use Photonic::Utils qw(lentzCF);
+use List::Util qw(min);
 use Moose;
 use MooseX::StrictConstructor;
 
@@ -136,7 +137,7 @@ has 'haydock' =>(is=>'ro', isa=>'Photonic::WE::S::AllH', required=>1);
 has 'nhActual'=>(is=>'ro', isa=>'Num', init_arg=>undef,
                  writer=>'_nhActual');
 has 'converged'=>(is=>'ro', isa=>'Num', init_arg=>undef, writer=>'_converged');
-has 'Gpp'=>(is=>'ro', isa=>'PDL::Complex', init_arg=>undef,
+has 'Gpp'=>(is=>'ro', isa=>'Photonic::Types::PDLComplex', init_arg=>undef,
 	    lazy=>1, builder=>'_build_Gpp',
 	      documentation=>'Value of projected Greens function');
 
@@ -144,40 +145,17 @@ sub _build_Gpp {
     my $self=shift;
     $self->haydock->run unless $self->haydock->iteration;
     my $epsR=$self->haydock->epsilonR;
-    my $as=$self->haydock->as;
-    my $bcs=$self->haydock->bcs;
-    # Continued fraction evaluation: Lentz method
-    # Numerical Recipes p. 171
-    my $tiny=1.e-30;
-    my $converged=0;
+    my $as=pdl(map r2C($_), @{$self->haydock->as})->cplx;
+    my $bcs=pdl(map r2C($_), @{$self->haydock->bcs})->cplx;
+    my $min= min($self->nh, $self->haydock->iteration);
     #    b0+a1/b1+a2/...
     #	lo debo convertir a
     #       1-a_0-g0g1b1^2/1-a1-g1g2b2^2/...
     #   entonces bn->1-an y an->-g{n-1}gnbn^2 o -bc_n
-    my $fn=1-$as->[0];
-    $fn=r2C($tiny) if $fn->re==0 and $fn->im==0;
-    my $n=1;
-    my ($fnm1, $Cnm1, $Dnm1)=($fn, $fn, r2C(0)); #previous coeffs.
-    my ($Cn, $Dn); #current coeffs.
-    my $Deltan;
-    while($n<$self->nh && $n<$self->haydock->iteration){
-	$Dn=1-$as->[$n]-$bcs->[$n]*$Dnm1;
-	$Dn=r2C($tiny) if $Dn->re==0 and $Dn->im==0;
-	$Cn=1-$as->[$n]-$bcs->[$n]/$Cnm1;
-	$Cn=r2C($tiny) if $Cn->re==0 and $Cn->im==0;
-	$Dn=1/$Dn;
-	$Deltan=$Cn*$Dn;
-	$fn=$fnm1*$Deltan;
-	last if $converged=$Deltan->approx(1, $self->smallE)->all;
-	$fnm1=$fn;
-	$Dnm1=$Dn;
-	$Cnm1=$Cn;
-	$n++;
-    }
+    my ($fn, $n)=lentzCF(1-$as, -$bcs, $min, $self->smallE);
     #If there are less available coefficients than $self->nh and all
     #of them were used, there is no remaining work to do, so, converged
-    $converged=1 if $self->haydock->iteration < $self->nh;
-    $self->_converged($converged);
+    $self->_converged($n<$min || $self->haydock->iteration<=$self->nh);
     $self->_nhActual($n);
     my $g0b02=$self->haydock->gs->[0]*$self->haydock->b2s->[0];
     return $g0b02/($epsR*$fn);
